#include <WiFi.h>
#include "time.h"
#include <iostream> 
#include <string> 
using namespace std; 
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <esp_now.h>
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

const char* ssid       = "JIMLIM";
const char* password   = "1234567890";

const char* ssid2 = "Filmboard";
const char* password2 = "FilmFilm";

const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = 21600;
const int   daylightOffset_sec = 3600;

const char* servermode = "http://192.168.4.1/modes";
const char* mode_pump = "http://192.168.4.1/mode_pump";
const char* humidity = "http://192.168.4.1/humiditys";
const char* duration = "http://192.168.4.1/Duration";
const char* hour = "http://192.168.4.1/hour";
const char* minutes = "http://192.168.4.1/minutes";

struct tm timeinfo;
const int RELAY_PIN = 0;
int actual_hour;
int actual_minutes;


void printLocalTime()
{ 
  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time////////////////////////////////");
    return;
  }
  int* hour_real = &timeinfo.tm_hour;
  int* minute_real = &timeinfo.tm_min;
  int* sec = &timeinfo.tm_sec;
  int actual_hour = *hour_real;
  int actual_minutes = *minute_real; 
  Serial.printf("%d:%d:%d RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR\n", *hour_real,*minute_real,*sec);

}

void setup()
{
  pinMode(RELAY_PIN, OUTPUT);
  Serial.begin(115200);

  //connect to WiFi
  Serial.printf("Connecting to %s ", ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
  }
  Serial.println(" CONNECTED Wifi \n");

  //init and get the time
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  printLocalTime();

  //disconnect WiFi as it's no longer needed
  WiFi.disconnect(true);
  WiFi.mode(WIFI_OFF);
  Serial.printf("Disconnected Wifi \n");

  //////////////////////////////////////////////////////////////////////

  WiFi.begin(ssid2, password2);
  Serial.println("Connecting");
  while(WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to WiFi network with IP Address: ");
  Serial.println(WiFi.localIP());

  //////////////////////////////////////////////////////////////////////
  
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3D for 128x64
    Serial.println(F("SSD1306 allocation failed"));
    for(;;);
  }
  delay(2000);
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(0, 10);

  display.println("start program");
  display.display();
}

void loop()
{
  digitalWrite(RELAY_PIN, HIGH);
  printLocalTime();
  if(WiFi.status()== WL_CONNECTED )
  {
      String servermode_value = httpGETRequest(servermode);
      String mode_pump_value = httpGETRequest(mode_pump);
      String humidity_value = httpGETRequest(humidity);
      String duration_value = httpGETRequest(duration);
      String hour_value = httpGETRequest(hour);
      String minutes_value = httpGETRequest(minutes);
      Serial.printf("mode = %s, pump = %s, humidity = %s, duration = %s, hour = %s, minutes = %s",
      servermode_value,mode_pump_value,humidity_value,duration_value,hour_value,minutes_value);
      Serial.printf("pump: active \n");
      Serial.printf("pump duration: %s \n",duration_value);
      Serial.println("mode: Set Manual Time");
      if(servermode_value == "MODE 1")
      {
        Serial.print("If mode 1\n");
        display.clearDisplay();
        display.setCursor(0, 10);
        display.println("pump: off");
        display.print("pump duration:");
        display.println(duration_value);
        display.println("mode: Set Manual Time");
        display.display();
        digitalWrite(RELAY_PIN, HIGH);  
        if(mode_pump_value == "1")
          {
          display.clearDisplay();
          display.setCursor(0, 10);
          display.println("pump: on");
          display.print("pump duration:");
          display.println(duration_value);
          display.println("mode: Set Manual Time");
          display.display();
          digitalWrite(RELAY_PIN, LOW);
          
          while(mode_pump_value == "1"){
            mode_pump_value = httpGETRequest(mode_pump);
            Serial.printf("mode 1 loop, mode pump :%s \n",mode_pump_value);
            delay(500);}
          display.clearDisplay();
          display.setCursor(0, 10);
          display.println("pump: off");
          display.print("pump duration:");
          display.println(duration_value);
          display.println("mode: Set Manual Time");
          display.display();
          digitalWrite(RELAY_PIN, HIGH);
          
        }
        int hour_value_int = atoi(hour_value.c_str());
        int minutes_value_int = atoi(minutes_value.c_str());
        actual_hour = *&timeinfo.tm_hour;
        actual_minutes = *&timeinfo.tm_min;
        int actual_sec = *&timeinfo.tm_sec;
        Serial.printf("hour_value_int = %d, Actual_hour = %d \nminutes_value_int = %d, Actual_minutes = %d \n",hour_value_int,actual_hour,minutes_value_int, actual_minutes);
        Serial.printf("second = %d \n", *&timeinfo.tm_sec);
        if(actual_hour == hour_value_int && actual_minutes == minutes_value_int && actual_sec == 59)
        {
          Serial.printf("In TIMEEEEEEEEEEEEEEEEEEEEEEE");
          display.clearDisplay();
          display.setCursor(0, 10);
          display.println("pump: on");
          display.print("pump duration:");
          display.println(duration_value);
          display.println("mode: Set Manual Time");
          display.display();
          digitalWrite(RELAY_PIN, LOW);
          int duration_value_int = atoi(duration_value.c_str());
          Serial.printf("duration_value_int = %d \n",duration_value_int);
          delay(duration_value_int*1000);
          display.clearDisplay();
          display.setCursor(0, 10);
          display.println("pump: off");
          display.print("pump duration:");
          display.println(duration_value);
          display.println("mode: Set Manual Time");
          display.display();
          digitalWrite(RELAY_PIN, HIGH);
        }
      }
      else
      {
        Serial.println("mode: Humidity");
        if(mode_pump_value == "1")
        {
          display.clearDisplay();
          display.setCursor(0, 10);
          display.println("pump: on");
          display.print("pump duration:");
          display.println(duration_value);
          display.println("mode: Humidity");
          display.display();
          digitalWrite(RELAY_PIN, LOW);
          while(mode_pump_value == "1"){            
            mode_pump_value = httpGETRequest(mode_pump);
            Serial.printf("mode 2 loop, mode pump :%s \n",mode_pump_value);
            delay(500);}
          display.clearDisplay();
          display.setCursor(0, 10);
          display.println("pump: off");
          display.print("pump duration:");
          display.println(duration_value);
          display.println("mode: Humidity");
          display.display();
          digitalWrite(RELAY_PIN, HIGH);
        }
        Serial.printf("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n");
        int humidity_value_int =  atoi(humidity_value.c_str());
        Serial.printf("humidity value :%d \n",humidity_value_int);
        if(humidity_value_int < 3000)
        {
          display.clearDisplay();
          display.setCursor(0, 10);
          display.println("pump: on");
          display.print("pump duration:");
          display.println(duration_value);
          display.println("mode: Humidity");
          display.display();
          digitalWrite(RELAY_PIN, LOW);
          Serial.printf("humidity value :%d \n",humidity_value_int);
          while(humidity_value_int < 3000 && servermode_value == "MODE 2"){
            humidity_value = httpGETRequest(humidity);
            servermode_value = httpGETRequest(servermode);
            humidity_value_int = atoi(humidity_value.c_str());
            Serial.printf("mode 2 loop, humidity value :%d \n",humidity_value_int);
            delay(500);}
          display.clearDisplay();
          display.setCursor(0, 10);
          display.println("pump: off");
          display.print("pump duration:");
          display.println(duration_value);
          display.println("mode: Humidity");
          display.display();
          digitalWrite(RELAY_PIN, HIGH);
        }
      }
      
    
  }
  else 
  {
    Serial.println("WiFi Disconnected");
  }
  
}




//////////////////////////////////////////////////////////////////////////////
String httpGETRequest(const char* serverName) {
  HTTPClient http;
    
  // Your IP address with path or Domain name with URL path 
  http.begin(serverName);
  
  // Send HTTP POST request
  int httpResponseCode = http.GET();
  
  String payload = "--"; 
  
  if (httpResponseCode>0) {
    Serial.print("HTTP Response code: ");
    Serial.println(httpResponseCode);
    payload = http.getString();
  }
  else {
    Serial.print("Error code: ");
    Serial.println(httpResponseCode);
  }
  // Free resources
  http.end();

  return payload;
}